/*	$NetBSD$	*/

/*-
 * Copyright (c) 2001 Minoura Makoto.
 * Copyright (c) 1994 Charles M. Hannum.
 * Copyright (c) 1992, 1993 Erik Forsberg.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL I BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Synaptics TouchPad via PS/2 aux port.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/device.h>
#include <sys/ioctl.h>

#include <machine/bus.h>

#include <dev/ic/stpreg.h>
#include <dev/ic/stpvar.h>

#include <dev/wscons/wsconsio.h>
#include <dev/wscons/wsmousevar.h>

static char UNK[] = "Unknown sensor type";
static char *stp_idstring[] = {
	UNK,
	"TM41xx134 Standard TouchPad",
	"TM41xx156 Mini module",
	"TM41xx180 Super module",
	UNK, UNK, UNK, UNK,
	"TM41xx220 Ultra-thin module",
	"TM41xx230 Wide pad module",
	UNK,
	"TM41xx240 Stamp pad module",
	"TM41xx140 SubMini module",
	UNK, UNK,
	"TM41xx301 Advanced Technology Pad",
	"TM41xx221 Ultra-thin module, connector reserved"
};
#define STP_NID		(sizeof(stp_idstring) / sizeof(stp_idstring[0]))

int stp_init (arg)
	void *arg;
{
	struct stp_softc *sc = arg;
	int r;
	stp_identifier ident;

	r = (sc->sc_identify) (arg, &ident);
	if (r) {
		printf("\nstp_init: identification failed\n");
		return EIO;
	}
	printf(": Synaptics TouchPad rev. %d.%d\n",
	       ident.si_info_major, ident.si_info_minor);

	r = ((ident.si_model_id & STP_MODELID_INFO_SENSOR_MASK)
	     >> STP_MODELID_INFO_SENSOR_SHIFT);
	printf("%s: %s", sc->sc_dev.dv_xname,
	       (r < STP_NID)?stp_idstring[r]:UNK);
	r = ((ident.si_model_id & STP_MODELID_INFO_GEOMETRY_MASK)
	     >> STP_MODELID_INFO_GEOMETRY_SHIFT);
	if (r == 1)
		printf(", rectangular");
	else if (r == 2)
		printf(", oval");
	if (ident.si_model_id & STP_MODELID_INFO_NEW_ABS)
		printf(", new packet");
	if (ident.si_model_id & STP_MODELID_CAP_PEN)
		printf(", pen capable");
	printf(".\n%s: setting relative mode.\n", sc->sc_dev.dv_xname);
	r = (sc->sc_setmode) (arg, STP_MODE_DISABLE_GESTURE);
	if (r) {
		printf("stp_init: setmode failed\n");
		return EIO;
	}

	return 0;
}

